#! /bin/bash
#mksrvvz.sh
#-------------------------------------------------------------------
#procedure to install a specific VZserver 
#$1 is the vserver name  (tokyo.safe.ca, etc..)
#$2 is the distribution level	(fc7 by default)
#$3 is the arch level	(i386)
#$4 base directory if specified
#-------------------------------------------------------------------
#declaration des RPM qui doivent etre installer
#-------------------------------------------------------------------
PREFIX="2001:4830:1153"
#domaine name
LOCDOM=`hostname -d`
#hostname
HOST=`hostname --short`
BASE=""
if [ ! -z "$4" ] ; then
  BASE=/$4
  fi
#-------------------------------------------------------------------
REPOLOC=$HOST-repo.$LOCDOM
DNS1=`grep nameserver /etc/vz/vztmpl/vzresolv.conf | cut -d' ' -f2 | head -1`

#installation base directory
VDIR=/etc/vz
. $VDIR/vz.conf		#getting local parameters
DSRVZ=/vz$BASE/$1
ETH0=eth0		#default veth interface
ETH1=eth1		#default veth IPV6  interface
BR0=br0			#default bridge interface
BR1=br1			#IPV6 Bridge
PUBVSR=$1.$LOCDOM
NAMVSR=`echo $1 | tr  '.' '_'`
NAMIP4=$NAMVSR"_ip4"
NAMIP6=$NAMVSR"_ip6"
IPNUM=`dig +short $PUBVSR | grep -v [a-z]`
#===================================================================
#setting distribution and Architecture.
DISTRIB="el5.5"
CPUARCH="i386"
if [ -n "$2" ] ; then
  DISTRIB=$2;
  fi
if [ -n "$3" ] ; then
  CPUARCH=$3;
  fi
#===================================================================
#procedure to compute VEID
getveid()

{
IP1=`echo $1 | cut -d'.' -f4`
IP2=`echo $1 | cut -d'.' -f3`
IP3=`echo $1 | cut -d'.' -f2`
VEID=`expr  '(' $IP3 '*' '1000000' ')' '+' '(' $IP2 '*' '1000' ')' '+' $IP1`
}
#===================================================================
#lock procedure
lockproc()

{
LOCKFILE=/var/lock/`basename $0`.$1.lck

trap "rm -f $LOCKFILE ; exit 1" 1 2 3 15
if (set -C; : > $LOCKFILE) 2> /dev/null ; then
    echo $$ >> $LOCKFILE
  else
    echo "Another '`basename $0` $1' is running, existing at once!"
    echo "remove $LOCKFILE, if it is not the case"
    rm -f $LOCKFILE
    exit 0;
  fi
}
#===================================================================
#procedure to get the vz template according the distribution
get_template()

{
export TPL="linux-$DISTRIB-$CPUARCH-vzserver"
}
#===================================================================
#procedure pour faire une installation de base
#===================================================================
if [ -z "$IPNUM" ] ; then
 echo "Unable to find IP for $PUBVSR"
 echo "exiting"
 rm -f $LOCKFILE
 exit 1 
 fi
lockproc $1
#===================================================================
#computing the VEID
getveid $IPNUM
PRIV=$DSRVZ/private
VZCTL=/usr/sbin/vzctl
if [ -d $PRIV ] ; then
  echo "$PUBVSR virtual server is already existing, must be removed first"
  echo "command to do:"
  echo "$VZCTL stop $NAMVSR; $VZCTL destroy $NAMVSR"
  echo "exiting"
  rm -f $LOCKFILE
  exit 0
  fi
get_template
echo "installing $PUBVSR IP=$IPNUM"
(
echo "Downloading template file: $TPL"
cd /vz/template/cache ; 
wget --quiet ftp://$REPOLOC/pub/linux/vztemplate/$TPL.tar.gz
)
mkdir -p $DSRVZ
$VZCTL --verbose create $VEID 			\
		--config $CONFIGFILE		\
	        --private $DSRVZ/private	\
	        --root $DSRVZ/root		\
		--ostemplate $TPL		\
		--config $TPL
rm -fr /vz/template/cache/$TPL.* &
$VZCTL set $VEID --name $NAMVSR			\
                --searchdomain $LOCDOM		\
		--netif_add $ETH0,,$NAMIP4,,$BR0\
		--netif_add $ETH1,,$NAMIP6,,$BR1\
                --nameserver $DNS1		\
		--features "nfs:off"		\
                --save
#===================================================================
#ajusting Network
(
echo "DEVICE=$ETH0"
echo "BOOTPROTO=static"
echo "ONBOOT=yes"
echo "IPADDR=$IPNUM"
echo "NETMASK=255.255.255.255"
) >  $PRIV/etc/sysconfig/network-scripts/ifcfg-$ETH0

(
echo "192.0.2.0/24 dev $ETH0 scope link"
echo "default via 192.0.2.1 dev $ETH0"
) >  $PRIV/etc/sysconfig/network-scripts/route-$ETH0

(
echo "DEVICE=$ETH1"
echo "ONBOOT=yes"
echo "IPV6INIT=yes"
echo "IPV6ADDR=$PREFIX::$IPNUM"
echo "IPV6_DEFAULTGW=$PREFIX::1"
) >  $PRIV/etc/sysconfig/network-scripts/ifcfg-$ETH1

#needed by RH8
(
cd  $PRIV/etc/sysconfig/networking/devices
ln -s ../../network-scripts/route-$ETH0 $ETH0.route
)
if [ "$DISTRIB" = "rh7.3" ] ; then
  (
  echo "#!/bin/sh"
  echo "#---------------"
  echo "#to setup the default route"
  echo "#---------------"
  echo ". /etc/sysconfig/network"
  echo "/sbin/route add default gw \${GATEWAY}"
  ) > $PRIV/sbin/ifup-local
  chmod 750 $PRIV/sbin/ifup-local
  (
  echo "$ETH0 net 192.0.2.0 netmask 255.255.255.0 dev"
  ) > $PRIV/etc/sysconfig/static-routes
  fi
#===================================================================
#ajusting password and user
if [ -f $PRIV/etc/shadow ] ; then
  cp $PRIV/etc/shadow $PRIV/etc/shadow.tmp
  (grep root /etc/shadow ; grep -v root $PRIV/etc/shadow.tmp) > \
     $PRIV/etc/shadow
  rm $PRIV/etc/shadow.tmp
  fi
#perparing root access
cp -a ~root/.ssh $PRIV/root
#adding root .ssh information to rpm skeleton /etc/skelrpm
mkdir -p $PRIV/etc/skelrpm
cp -a ~safe/.ssh $PRIV/etc/skelrpm

#adding safe sudoers priviledge
cat >> $PRIV/etc/sudoers << !EOT
#-----------------------------------------------------------
#adding safe sudoers priviledge
%safe	ALL = NOPASSWD: /usr/sbin/useradd, /usr/sbin/userdel, /bin/mknod
#-----------------------------------------------------------
!EOT

#===================================================================
#inserting debug in syslog and rsyslog
(
echo "#----------------------------------------------------------------"
echo "#Regulus debug"
echo -e "*.=debug\t\t\t\t\t/var/log/debug"
echo "#----------------------------------------------------------------"
)       >> $PRIV/tmp/$$.debug
for sys in syslog rsyslog
  do
  if [ -f $PRIV/etc/$sys.conf ] ; then
    cat $PRIV/tmp/$$.debug >> $PRIV/etc/$sys.conf
    /usr/sbin/chroot $PRIV /sbin/chkconfig $sys on
    pidfile=`echo -n $sys; echo d`
    (
    echo "/var/log/debug {"
    echo -e "    sharedscripts"
    echo -e "    postrotate"
    echo -e "\t/bin/kill -HUP \`cat /var/run/$pidfile.pid 2> /dev/null\` 2> /dev/null || true"
    echo -e "    endscript"
    echo "    }"
     )       >> $PRIV/etc/logrotate.d/debug
    fi
  break;
  done
rm -f $PRIV/tmp/$$.debug

#===================================================================
#adjusting selinux
mkdir -p $PRIV/etc/selinux
(
echo "# This file controls the state of SELinux on the system."
echo "# SELINUX= can take one of these three values:"
echo "#       enforcing - SELinux security policy is enforced."
echo "#       permissive - SELinux prints warnings instead of enforcing."
echo "#       disabled - SELinux is fully disabled."
echo "SELINUX=disabled"
echo "# SELINUXTYPE= type of policy in use. Possible values are:"
echo "#       targeted - Only targeted network daemons are protected."
echo "#       strict - Full SELinux protection."
echo "SELINUXTYPE=targeted"
echo ""
echo "# SETLOCALDEFS= Check local definition changes"
echo "SETLOCALDEFS=0"
)	>> $PRIV/etc/selinux/config
#===================================================================
#adjusting yum.conf
(
echo "#-----------------------------------------------"
echo "#to avoid probing external site"
echo "enabled=0"
echo "#-----------------------------------------------"
) >> $PRIV/etc/yum.conf
#adding downgrade_ftp to fastestmirror.conf
if [ -f $PRIV/etc/yum/pluginconf.d/fastestmirror.conf ] ; then
  (
  echo "#--------------------------------------------"
  echo "#to check ftp site for site too"
  echo "downgrade_ftp=false"
  echo "#--------------------------------------------"
  ) >> $PRIV/etc/yum/pluginconf.d/fastestmirror.conf
  fi
#pour n'utiliser que le saferepo de SAFE
FEDREPO="fedora-rawhide.repo fedora-updates-testing.repo"
FEDREPO=$FEDREPO" fedora-updates.repo fedora.repo"
for file in $FEDREPO
  do
  repofile=$PRIV/etc/yum.repos.d/$file 
  if [ -f $repofile ] ; then
    sed -i -e "s/enabled=1/enabled=0/" $repofile
    fi
  done
#===================================================================
#ajusting /etc/hosts
(
echo "#Done by `basename $0`"
echo -e "127.0.0.1\t\tlocalhost.localdomain\t\tlocalhost"
echo "#----------------------------------------------------------------"
echo -e "$IPNUM\t\t$PUBVSR\t\t$1"
echo -e "$PREFIX::$IPNUM\t\t$PUBVSR\t\t$1"
echo "#----------------------------------------------------------------"
)       >> $PRIV/etc/hosts
#===================================================================
#constant SSH information
cp /etc/vzsafe/ssh_info/* $PRIV/etc/ssh
#ajusting networking information
(
echo "#Done by `basename $0`"
echo "NETWORKING=\"yes\""
echo "NETWORKING_IPV6=yes"
echo "#GATEDEV=\"venet0\""
echo "GATEWAY=\"192.0.2.1\""
echo "HOSTNAME=$PUBVSR"
) >> $PRIV//etc/sysconfig/network
#===================================================================
#setting the same constant system key for all systems
(
echo "ListenAddress $IPNUM"
echo "ListenAddress $PREFIX::$IPNUM"
echo "HostKey /etc/ssh/safe.ssh_host_key"
echo "HostKey /etc/ssh/safe.ssh_host_rsa_key"
echo "HostKey /etc/ssh/safe.ssh_host_dsa_key"
echo ""
) >> $PRIV/etc/ssh/sshd_config
#===================================================================
#adjustment after container start
if [ "$DISTRIB" = "mdv2010.1" ] ; then
  echo "search $LOCDOM" >> $PRIV/etc/resolvconf/resolv.conf.d/base
  echo "nameserver $DNS1" >> $PRIV/etc/resolvconf/resolv.conf.d/base
  fi 
#===================================================================
#starting the vserver
$VZCTL start $VEID 
sleep 10
#restarting network
$VZCTL exec $VEID  service network restart
#===================================================================
#adjusting iptables
if [ -f /etc/vzsafe/vziptables ] ; then
  cp /etc/vzsafe/vziptables $PRIV/etc/sysconfig/iptables
  if [ -c /etc/init.d/iptables ] ; then
    $VZCTL exec $VEID  service iptables restart
    fi
  fi
if [ -f /etc/vzsafe/vzip6tables ] ; then
  cp /etc/vzsafe/vzip6tables $PRIV/etc/sysconfig/ip6tables
  if [ -c /etc/init.d/ip6tables ] ; then
    $VZCTL exec $VEID  service ip6tables restart
    fi
  fi
#===================================================================
#installing safe users
$VZCTL exec $VEID groupadd -r -g 300 safe
$VZCTL exec $VEID useradd -p* -r -m -k /etc/skelrpm -g safe -u 300 safe
$VZCTL exec $VEID useradd -p* -r -m -k /etc/skelrpm -g safe -u 301 jmp
$VZCTL exec $VEID useradd -p* -r -m -k /etc/skelrpm -g safe -u 302 diane
$VZCTL exec $VEID useradd -p* -r -m -k /etc/skelrpm -g safe -u 303 vincent
#===================================================================
#waiting network upand running
echo "Waiting for Network update"
sleep 10
#===================================================================
#adjusting postgresql configuration
$VZCTL exec $VEID chkconfig postgresql on
DBVERS=`$VZCTL exec $VEID psql --version | head -1 | cut -d' ' -f3 | cut -d '.' -f1,2 | tr -d '.'`
if [ "$DBVERS" -ge "83" ] ; then
  $VZCTL exec $VEID service postgresql initdb
  if [ -f $PRIV/var/lib/pgsql/data/postgresql.conf ] ; then
    (
    echo "#-------------------------------------------------------"
    echo "#configuration specific a SAFE"
    echo "#-------------------------------------------------------"
    echo "log_line_prefix = '%t'"
    echo "listen_addresses = '*'"
    echo "#log_autovacuum_min_duration"
    echo "# -1 disables, 0 logs all actions and their durations,"
    echo "# > 0 logs only actions running at least that time."
    echo "log_autovacuum_min_duration = 0"
    echo "#-------------------------------------------------------"
    ) >> $PRIV/var/lib/pgsql/data/postgresql.conf
    fi
  fi
$VZCTL exec $VEID service postgresql start
#===================================================================
#adjusting snmpd configuration
if [ -f $PRIV/etc/snmp/snmpd.conf ] ; then
  if [ -f /etc/vzsafe/snmpd.addon ] ; then
    cat /etc/vzsafe/snmpd.addon >> $PRIV/etc/snmp/snmpd.conf
    fi
  $VZCTL exec $VEID chkconfig snmpd on
  $VZCTL exec $VEID service snmpd start
  fi
#===================================================================
#adjusting mount configuration
if [ -f $PRIV/etc/fstab ] ; then
  mkdir $PRIV/nfsloc
  $VZCTL exec $VEID chown safe.safe /nfsloc
  if [ -f /etc/vzsafe/fstab.addon ] ; then
    cat /etc/vzsafe/fstab.addon >> $PRIV/etc/fstab
    fi
  (
  echo "#-----------------------------------------"
  echo "#to mounted need nfs partition"
  echo "mount -a"
  echo "#-----------------------------------------"
  ) >>  $PRIV/etc/rc.d/rc.local
  fi
#===================================================================
#copying local environement
cp -fp /etc/environment $PRIV/etc
#===================================================================
#loading saferepo
DDIST=`repovar.sh $DISTRIB DDIST`
VNUM=`repovar.sh $DISTRIB VNUM`
REPODIR=/linux/$DDIST/$VNUM/addon/$CPUARCH/
$VZCTL exec $VEID rpm -ihv				\
  http://$REPOLOC/$REPODIR/saferepo-current.noarch.rpm
if [ -x $PRIV/usr/bin/yum ] ; then
  $VZCTL exec $VEID /usr/bin/yum clean all
  fi
#===================================================================
#END
rm -f $LOCKFILE

